import numpy as np
from sympy.physics.mechanics import *

class Orbit:
    '''Base orbit class that is used to build circular and eliptical orbits
    frame: sympy.ReferenceFrame object
    center: point/trajectory of the attractor #todo rename?
    color: (R,G,B) tuple to be used when plotting trajectory and body'''

    def __init__(self, frame, center, color = (0, 0, 0)):
        self.frame = frame
        self.center = center
        self.color = color
        self.dim = 0

    def get_position(self, epoch) -> Vector:
        pass

    def get_trajectory(self, epoch: np.array, center = None, frame = None):
        '''Returns a 2D or 3D numpy array of results from get_position()
        over time'''
        # todo: fix array dims
        #if self.dim == 3:
        pos = np.zeros_like([epoch, epoch, epoch], dtype = float)
        #elif self.dim ==2:
        #    pos = np.zeros_like([epoch, epoch], dtype = float)

        if frame is None:
            # todo: better explanation of how sympy frames work here,
            # some way to add the possib. of plotting around earth
            frame = self.frame

        for idx, ti in enumerate(epoch):
            # calls get_position() for each time, subtract center position
            if center is None:
                vec = self.get_position(ti).to_matrix(frame)
            else:
                vec = (self.get_position(ti) -
                       center.get_position(ti)).to_matrix(frame)
            pos[:, idx] = vec[:]
        return pos





from spiffy.orbit import Orbit
from numpy import cos, sin, linspace, pi
from spiffy.plotter import Plotter
from sympy.physics.mechanics import *


class CircularOrbit3D(Orbit):
    """Model of a ideal 3D orbit"""

    def __init__(self, parentFrame, center = None,
                 radius: float = 0, omega: float = 0, inclination: float = 0,
                 phase: float = 0, ascNode: float = 0, color = (0, 0, 0),
                 opt = 'sun'):
        super().__init__(parentFrame, center, color)
        self.frame = parentFrame.orientnew(
                'frame', 'Space', (inclination, 0, ascNode), 123)
        # todo: orient new frame with 'Space'
        # todo: fix frame naming scheme
        self.radius = radius

        self.phase = phase  # todo: rename parm
        self.ascNode = ascNode  # todo: rename parm
        self.dim = 3
        if opt == 'sun':
            self.mu = 1.327e20
        elif opt == 'earth':
            self.mu = 3.986004418e14
        self.period = self.get_period()

    def get_position(self, epoch, frame = None):
        '''given an epoch, returns a sympy vector for position'''
        # todo pass other frame argument
        r = self.radius

        ph = self.phase
        if frame is None:
            frame = self.frame

        x = r * cos((2*pi/self.period)*epoch + ph) * frame.x
        y = r * sin((2*pi/self.period)*epoch + ph) * frame.y
        z = 0 * frame.z
        if self.center is None:
            return x + y + z
        else:
            return x + y + z + self.center.get_position(epoch).express(frame)

    def get_position_matrix(self, epoch, frame):
        """Returns matrix instead of Vector to be used w/ Plotter"""
        return self.get_position(epoch).to_matrix(frame)

    def get_direction_matrix(self, other, epoch, frame):
        # print("get dir:")
        # print(self)
        se = self.get_position_matrix(epoch, frame)
        # print(other)
        ot = other.get_position_matrix(epoch, frame)
        direction = ot - se
        return direction

    def get_period(self):
        T = 2 * pi * (self.radius ** 3.0/self.mu) ** .5
        return T


class EllipticalOrbit3D(Orbit):
    """Model of a ideal 3D elliptical orbit"""

    def __init__(self, parentFrame, center = None,
                 eccentricity: float = 0, semiMajorAxis: float = 0,
                 inclination: float = 0, ascNode: float = 0,
                 periArg: float = 0, trueAnomaly: float = 0,
                 color = (0, 0, 0), opt = 'sun'):
        super().__init__(parentFrame, center, color)
        self.eccentricity = eccentricity
        self.semiMajorAxis = semiMajorAxis
        self.inclination = inclination
        self.ascNode = ascNode
        self.periArg = periArg
        self.trueAnomaly = trueAnomaly
        self.frame = parentFrame.orientnew(
                'frame', 'Space', (inclination, 0, ascNode), 123)
        # todo: orient new frame with 'Space'
        # todo: fix frame naming scheme
        self.dim = 3
        if opt == 'sun':
            self.mu = 1.327e20
        elif opt == 'earth':
            self.mu = 3.986004418e14
        self.period = self.get_period()

    def get_position(self, epoch, frame = None):
        '''given an epoch, returns a sympy vector for position'''
        # todo pass other frame argument
        e = self.eccentricity
        a = self.semiMajorAxis
        i = self.inclination
        Om = self.ascNode
        w = self.periArg
        f = self.trueAnomaly
        if frame is None:
            frame = self.frame

        r = a * (1.0 - e ** 2.0) / (1 + e * cos(2 * pi * epoch / self.period))
        x = r * cos(2 * pi * epoch / self.period + self.periArg) * frame.x
        y = r * sin(2 * pi * epoch / self.period + self.periArg) * frame.y
        z = 0 * frame.z
        if self.center is None:
            return x + y + z
        else:
            return x + y + z + self.center.get_position(epoch).express(frame)

    def get_position_matrix(self, epoch, frame):
        """Returns matrix instead of Vector to be used w/ Plotter"""
        return self.get_position(epoch).to_matrix(frame)

    def get_period(self):
        T = 2 * pi * (self.semiMajorAxis ** 3.0/self.mu) ** .5
        return T


if __name__ == '__main__':
    # testing the orbit and plotter classes
    N = ReferenceFrame('N')
    t = linspace(0, 10 * 24 * 3600, 5000)  # numpy array

    # creates orbits. Two objects orbiting a main body
    earth_orbit = CircularOrbit3D(N, radius = 1.5e11, color = (0, .6, .20))
    sat1_orbit = EllipticalOrbit3D(N, center = earth_orbit, semiMajorAxis = 1e9,
                                   eccentricity = .9, color = (1, 0, 0), opt = 'earth')
    #sat1_orbit = EllipticalOrbit3D(N, center = None, semiMajorAxis = 1e9,
    #                               eccentricity = .9, periArg = 0, opt = 'earth')
    sat2_orbit = EllipticalOrbit3D(N, center = earth_orbit, semiMajorAxis = 1e9,
                                   eccentricity = .9, periArg = (120 / 180) * pi,
                                   color = (0, 1, 0), opt = 'earth')
    sat3_orbit = EllipticalOrbit3D(N, center = earth_orbit, semiMajorAxis = 1e9,
                                   eccentricity = .9, periArg = (240 / 180) * pi,
                                   inclination = 15*(pi/180),
                                   color = (0, 0, 1), opt = 'sun')
    orbits = [earth_orbit, sat1_orbit, sat2_orbit, sat3_orbit]#, sat2_orbit, sat3_orbit]

    for obt in orbits:
        Plotter.plot_trajectory(obt.get_trajectory(t, frame = N), color = obt.color,
                                tube_radius = None)

    Plotter.show3d()  # calls matplotlib show


from spiffy.orbit3D import *
from spiffy.orbit2D import *
from numpy import linspace, pi


if __name__ == '__main__':
    # testing the orbit and plotter classes
#    mlab.figure(size = (800, 600))

    N = ReferenceFrame('N')
    t = linspace(0, .001, 10)  # numpy array
    #ti = 0.4
    # creates 3 orbits. Two objects orbiting a main body
    earth_orbit = CircularOrbit3D(N, None, 10, 1, 0, color = (0.1, 0.1, 0.1))
    sat1_orbit = CircularOrbit3D(N, earth_orbit, radius = 1, omega = 1, inclination = 5*(pi/180), ascNode = 0*(pi/180), phase = (0+180)*(pi/180), color = (1, 0, 0))
    sat2_orbit = CircularOrbit3D(N, earth_orbit, radius = 1, omega = 1, inclination = 5*(pi/180), ascNode = 120*(pi/180), phase = (-120+60)*(pi/180), color = (0, 1, 0))
    sat3_orbit = CircularOrbit3D(N, earth_orbit, radius = 1, omega = 1, inclination = 5*(pi/180), ascNode = 240*(pi/180), phase = (-240-60)*(pi/180), color = (0, 0, 1))

    orbits = [earth_orbit, sat1_orbit, sat2_orbit, sat3_orbit]
    #for i, ti in enumerate(t):
    #i = 1
    #ti = 0.4
    for i in range(len(t)):
        ti = t[i]
        for obt in orbits:
            Plotter.plot_trajectory(obt.get_trajectory(t, frame = N),
                                    tube_radius = None, color = obt.color,
                                    opacity = 0.2)
            Plotter.plot_point(obt.get_position_matrix(ti, N),
                               scale_factor = 0.02, color = obt.color)

        sat1_pos = sat1_orbit.get_position_matrix(ti, N)
        sat2_pos = sat2_orbit.get_position_matrix(ti, N)
        sat3_pos = sat3_orbit.get_position_matrix(ti, N)
        avg_pos = (sat1_pos + sat2_pos + sat3_pos)/3

        Plotter.plot_separation(sat1_pos, sat2_pos,
                                color = (0.8, 0.8, 0.8), tube_radius = None,)
        Plotter.plot_separation(sat1_pos, sat3_pos,
                                color = (0.8, 0.8, 0.8), tube_radius = None,)
        Plotter.plot_separation(sat2_pos, sat3_pos,
                                color = (0.8, 0.8, 0.8), tube_radius = None,)
        Plotter.plot_point([0, 0, 0], scale_factor = .5, color = (0.8,.8,0.3))
        Plotter.plot_point(avg_pos, scale_factor = 0.01, color = (0.8,0.8,0.8))
        plane_vec1 = sat1_orbit.get_direction_matrix(sat2_orbit, ti, N)
        plane_vec2 = sat1_orbit.get_direction_matrix(sat3_orbit, ti, N)
        #print("main:")
        #print(sat1_orbit.get_position_matrix(0.1, N))
        #print(sat2_orbit.get_position_matrix(0.1, N))
        cr = plane_vec1.cross(plane_vec2)
        #print(plane_vec)
        Plotter.plot_vector(avg_pos, cr,  color = (0.8,0.8,0.8),
                            scale_factor = 4)
        filename = "test_cartwheel" + str(i) + ".png"

        #mlab.view(distance = 4, elevation = 80)
        Plotter.show3d()  # calls mayavi show
        #mlab.savefig(filename)
        #mlab.clf()


import numpy as np
import mayavi.mlab as mlab
import os
import sys

if __name__ == '__main__':
    path = os.path.dirname(__file__)

    print(path)
    with open(os.path.join(path,'meshplottercfg.txt'),'r') as cfgfile:
        for line in cfgfile:
            if 'PHI_MIN' in line:
                PHI_MIN = int(line.split('=')[1])
            elif 'PHI_MAX' in line:
                PHI_MAX = int(line.split('=')[1])
            elif 'DELTA_PHI' in line:
                DELTA_PHI = int(line.split('=')[1])
            elif 'THETA_MIN' in line:
                THETA_MIN = int(line.split('=')[1])
            elif 'THETA_MAX' in line:
                THETA_MAX = int(line.split('=')[1])
            elif 'DELTA_THETA' in line:
                DELTA_THETA = int(line.split('=')[1])

    phi, theta = np.meshgrid(np.arange(PHI_MIN,   PHI_MAX,   DELTA_PHI),
                             np.arange(THETA_MIN, THETA_MAX, DELTA_THETA))

    X_ = np.zeros(phi.shape)
    Y_ = np.zeros(phi.shape)
    Z_ = np.zeros(phi.shape)

    filename = sys.argv[1]

    with open(filename, 'r') as file:
        print('Importando arquivo...')
        for line in file:
            #if i % 10 == 0:
            s = line.split()
            x = float(s[0])
            y = float(s[1])
            z = float(s[2])
            p = float(s[3])
            t = float(s[4])
            print(p)
            print(t)
            try:
                i = list(phi[0,:]).index(int(round(p)))
                j = list(theta[:,0]).index(int(round(t)))

                X_[i, j] = x
                Y_[i, j] = y
                Z_[i, j] = z
            except:
                pass

    print('Plotando.')
    mlab.mesh(X_, Y_, Z_)
    mlab.show()


import mayavi.mlab as mlab
import numpy as np
import matplotlib.pyplot as mtplt

class Plotter:
    """Class to gather plotting functionalities.
    Interfaces with mayavi functions"""
    def __init__(self):
        pass

    @staticmethod
    def plot_trajectory(trajectory, **kwargs):
        """Plots curve from trajectory (3d x time numpy array).
        Returns plot3d object"""
        plot = mlab.plot3d(trajectory[0, :], trajectory[1, :], trajectory[2, :],
                      **kwargs)
        return plot

    @staticmethod
    def plot_trajectory2d(trajectory, **kwargs):
        """Plots curve from trajectory (2d x time numpy array).
        Returns plot2d object"""
        plot = mtplt.plot(trajectory[0, :], trajectory[1, :])
        return plot

    @staticmethod
    def plot_separation(pos1, pos2, **kwargs):
        """Plots straight line between 2 points"""
        # todo: better way of dealing with np.arrays needed by mayavi
        plot = mlab.plot3d([np.double(pos1[0]), np.double(pos2[0])],
                      [np.double(pos1[1]), np.double(pos2[1])],
                      [np.double(pos1[2]), np.double(pos2[2])],
                      **kwargs)
        return plot

    @staticmethod
    def plot_vector(pos, direction, **kwargs):
        """Plots arrow at pos, pointing w/ direction"""

        # todo: guarantee normalized dir? and/or produce dir from 2 positions
        plot = mlab.quiver3d([np.double(pos[0])],
                        [np.double(pos[1])],
                        [np.double(pos[2])],
                        [np.double(direction[0])],
                        [np.double(direction[1])],
                        [np.double(direction[2])],
                        **kwargs)
        return plot

    @staticmethod
    def plot_point(pos, **kwargs):
        """PLots ball to represent body"""

        # todo: special visualization cases for Sun and planets?
        plot = mlab.points3d(np.double(pos[0]),
                        np.double(pos[1]),
                        np.double(pos[2]),
                        **kwargs)
        return plot

    @staticmethod
    def plot_surface(points, **kwargs):
        """Plots surface from points (3d numpy array).
        Returns plot3d object"""
        plot = mlab.contour3d(points[0], points[1], points[2])
        return plot

    @staticmethod
    def show3d():
        return mlab.show()

    @staticmethod
    def show2d():
        mtplt.axes().set_aspect('equal', 'datalim')
        return mtplt.show()
